package com.ruoyi.ss.device.service;

import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.dto.DeviceBatchUpdateModelDTO;
import com.ruoyi.ss.device.domain.dto.DeviceRegisterDTO;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;

/**
 * 设备Service接口
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
public interface DeviceService
{
    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    public DeviceVO selectSmDeviceByDeviceId(Long deviceId);

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备集合
     */
    public List<DeviceVO> selectSmDeviceList(DeviceQuery smDevice);

    /**
     * 新增设备
     *
     * @param device 设备
     * @return 结果
     */
    public int insertSmDevice(Device device);

    /**
     * 修改设备
     *
     * @param device 设备
     * @return 结果
     */
    public int updateSmDevice(DeviceBO device);

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键集合
     * @return 结果
     */
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 条件查询数量
     * @param dto 查询条件
     * @return 结果
     */
    int selectCount(DeviceQuery dto);

    /**
     * 查询指定店铺下的设备列表
     * @param storeId
     */
    List<DeviceVO> selectSmDeviceByStoreId(Long storeId);

    /**
     * 增加时间
     *
     * @param deviceId 设备id
     * @param seconds  时长（秒）
     * @param withIot  是否充值物联网设备
     */
    boolean addTime(Long deviceId, long seconds, boolean withIot);

    /**
     * 逻辑删除
     */
    boolean logicDel(List<Long> deviceIds);

    /**
     * 通过型号id列表，查询设备列表，并分组
     * @param modelIds 型号id列表
     */
    Map<Long, List<DeviceVO>> selectGroupByModelIds(List<Long> modelIds);

    /**
     * 切换设备通电状态（通/断电）
     * @param deviceId 设备id
     * @param status 通电状态
     */
    boolean changePowerStatus(Long deviceId, DevicePowerStatus status);

    /**
     * 拉取设备信息，并保存到数据库
     * @param deviceIds 设备id列表（为空则查询所有）
     */
    void pullDeviceInfo(List<Long> deviceIds);

    /**
     * 通过SN绑定设备
     *
     * @param storeId  店铺ID
     * @param userId
     * @param deviceNo 设备编号（SN）
     */
    String bindBySn(Long storeId, Long userId, String deviceNo);

    /**
     * 通过MAC绑定设备
     */
    String bindByMac(Long storeId, Long userId, String mac);

    /**
     * 商户解绑设备
     *
     * @param deviceId 设备id
     */
    int unbind(Long deviceId);

    /**
     * 通过mac查询设备
     * @param mac mac号
     * @return 设备信息
     */
    DeviceVO selectByMac(String mac);

    /**
     * 归零电表，并关闭订单
     * @param deviceId 设备id
     */
    boolean resetTimeWithBill(Long deviceId);

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    int bindCodeByMac(String mac);

    /**
     * ids查询列表
     */
    default List<DeviceVO> selectListByIds(List<Long> ids) {
        return selectListByIds(ids, null);
    }

    /**
     * ids查询列表
     */
    List<DeviceVO> selectListByIds(List<Long> ids, Long userId);

    /**
     * 查询各店铺的设备数量,并分组
     * @param storeIds 店铺id
     */
    Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds);

    /**
     * 条件查询数量，并分组
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    <K> Map<K, Integer> selectCommonCountMap(DeviceQuery query, Function<? super DeviceCountVO, ? extends K> keyMapper );

    /**
     * 获取设备，包含套餐列表
     * @param deviceNo 设备编码
     * @return
     */
    DeviceVO getDeviceWithSuitList(String deviceNo);

    /**
     * 根据设备编号查询
     */
    DeviceVO selectByDeviceNo(String deviceNo);


    /**
     * 设备绑定SN码
     * @param deviceId 设备ID
     * @param sn SN
     */
    int bindSn(Long deviceId, String sn);

    /**
     * 根据SN码查询设备是否已经绑定
     *
     * @param deviceNo
     * @return
     */
    int bindCodeBySn(String deviceNo);

    /**
     * 更新设备服务费
     */
    int updateServiceRateByUserId(Long userId, BigDecimal serviceRate);

    /**
     * 更新设备信息
     */
    int updateByIot(Device device);

    /**
     * 刷新设备状态，如果过期，则修改为正常
     */
    void freshStatus(DeviceVO device);

    /**
     * 设备心跳，用于检测是否在线
     */
    void deviceHeartBeat();

    boolean addTimeByUser(Long deviceId, long seconds, boolean b, String reason);

    /**
     * 设备注册
     */
    int register(DeviceRegisterDTO dto);

    /**
     * 获取所有MAC
     */
    List<String> selectMacList(DeviceQuery query);

    /**
     * 查询一个
     */
    DeviceVO selectOne(DeviceQuery query);

    /**
     * 获取当前用户正在使用中的设备
     */
    List<DeviceVO> selectUsingDevice(Long userId);

    /**
     * 查询设备使用中的用户ID
     *
     * @param deviceId 设备ID
     * @return 使用中的用户ID
     */
    List<Long> selectDeviceUsingUser(Long deviceId);

    /**
     * 开关设备
     */
    boolean switchDevice(Long deviceId, boolean open);

    /**
     * 批量更新设备型号
     */
    int batchUpdateModel(DeviceBatchUpdateModelDTO dto);

    /**
     * 更新服务费
     */
    int updateServiceRate(DeviceVO data);

    /**
     * 续费时长
     * @param deviceId 设备ID
     * @param time 时长
     * @param timeUnit 时长单位
     */
    int renewalRentTime(Long deviceId, int time, TimeUnit timeUnit);

    /**
     * 刷新设备数据
     */
    void pullDeviceInfo(Long deviceId);

    /**
     * 清空设备时长、电量
     */
    int clearTimeAndEle(DeviceVO device, boolean required);

    /**
     * 设备归零时长
     */
    int resetTime(DeviceVO device, boolean required);

    /**
     * 设备归零电量
     */
    int resetEle(DeviceVO device, boolean required);

    /**
     * 归零电量
     */
    int resetEle(Long deviceId, boolean required);

    /**
     * 电量归零，并关闭订单
     */
    int resetEleWithBill(Long deviceId);

    int updateLastRecoverTime(Long deviceId, Long lastRecoverTime);

    /**
     * 设置时间
     * @param device 设备
     * @param expireTime 过期时间
     * @param withIot 是否操作设备
     * @param tryCount 尝试次数
     */
    int setTime(DeviceVO device, LocalDateTime expireTime, boolean withIot, int tryCount);

}
